import click
from sys import argv

from pywrdgen.password import Password

CONTEXT_SETTINGS = {
        'help_option_names': ['-h', '--help'],
    }


#
# Main entry-point. Handles global flags, and info blurb.
#
@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version='0.0.1')
@click.pass_context
@click.option('-p', '--password-only',
              default=False,
              is_flag=True,
              help='outut only generated passwords')
@click.option('-d', '--debug',
              default=False,
              is_flag=True,
              help='include extra output text')
def main(ctx, **kwargs):
    '''Generate (possibly) secure passwords.'''

    # Pull the **kwargs flags out and into our context object. This will
    # be passed into sub-commands so they may access these global flags.
    ctx.obj = {
            'password_only': kwargs['password_only'],
            'debug': kwargs['debug'],
        }

    # Return early and skip the info message if password_only flag is enabled.
    if ctx.obj['password_only']:
        return

    print('pywrdgen')
    print('--------')
    print('Generate (possibly) secure passwords. By default, there are no')
    print('flags enabled in `gen` mode. You must enable some of them for a')
    print('password to be generated. The combination of options selected')
    print('form the baseline for how secure the generated password is.')
    print()
    print('There is no guarantee that any given generated password will be')
    print('secure or fit for any purpose. The ratings given are determined')
    print('based on your selected combination of options. Enjoy!')
    print()

#
# Password List Generation. Generates, and displays a list of passwords
# generated based on the flags set on the command line.
#
@main.command()
@click.pass_context
@click.option('-a', '--alpha',
              default=False,
              is_flag=True,
              help='include lowercase alphabetic characters [a-z]')
@click.option('-u', '--upper',
              default=False,
              is_flag=True,
              help='include uppercase alphabetic characters [A-Z]')
@click.option('-n', '--numerals',
              default=False,
              is_flag=True,
              help='include numerals [0-9]')
@click.option('-s', '--special',
              default=False,
              is_flag=True,
              help='include special characters [LIST THEM HERE -- DYNAMICALLY]')
@click.option('-x', '--hexadecimal',
              default=False,
              is_flag=True,
              help='generate password in hexadecimal only [A-F0-9]')
@click.option('-l', '--length',
              default=0,
              type=int,
              help='length of password to generate, in characters')
@click.option('-c', '--count',
              default=0,
              type=int,
              help='number of passwords to generate, in total')
def gen(ctx, **kwargs):
    '''Generate a (possibly) secure password based on the specified options.'''
    if ctx.obj['debug']:
        for key, value in kwargs.items():
            print(f'{key}: {value}')
        print()

    # Create the new password object, providing it a **kwargs.
    passwords = []
    for i in range(kwargs['count']):
        p = Password(**kwargs)
        #p.generate()
        passwords.append(p)

    # Do some things to determine strength of password.
    # Based on options. Maybe also an algorithm to score it's randomness?

    # Output score, and (list of) passwords.
    if len(passwords) > 0:
        if not ctx.obj['password_only']:
            print(f'Generated {len(passwords)} passwords:')
        for password in passwords:
            print(password)
    # If no passwords generated
    else:
        # If we are *not* in password only mode, display an error message.
        if ctx.obj['password_only'] is False:
            print(
                'No password was generated. Please check the help text above.')
            print('The full list of generator flags is available with')
            print(f'{argv[0]} gen --help')


if __name__ == '__main__':
    main()



'''Represents a password that can be generated based on it's attributes.
'''


from random import choice

from string import (
    ascii_lowercase,
    ascii_uppercase,
    digits,
    punctuation,
    hexdigits,
)


class Password():
    def __init__(self, **kwargs):
        '''Create a new password object. Assign Password attributes based on
provided **kwargs, or defaults if missing.

  **kwargs:
    alpha -- include lowercase alphabetic characters [a-z]
    upper -- include uppercase alphabetic characters [A-Z]
    numerals -- include numerals [0-9]
    special -- include special characters ...'
    length -- total length, in characters, of generated password'
        '''
        self._password = None
        self.char_set = ''
        self.alpha = kwargs['alpha']
        self.upper = kwargs['upper']
        self.numerals = kwargs['numerals']
        self.special = kwargs['special']
        self.hexadecimal = kwargs['hexadecimal']
        self.length = kwargs['length']

    def __str__(self):
        '''Return the generated password.'''
        return self.password

    def __repr__(self):
        '''Return a list of attributes that can be provided as **kwargs into
a new Password object. Seed is not currently supported, so new passwords
will not be exactly the same.
        '''
        pass

    @property
    def password(self):
        '''Getter method for password. Return the already generated password,
if one exists. Otherwise, generate a new one and return that.
        '''
        if self._password is None:
            self.generate()
            return self._password
        else:
            return self._password

    def build_character_set(self):
        '''Assemble the list of acceptable characters for password generation.
        '''
        if self.hexadecimal:
            # If hexadecimal output enabled, return early.
            # Use *only* hex in character set.
            # Do some manipulations here to remove lowercase `a-f`
            if self.upper:
                hexcase = hexdigits.upper()
            else:
                hexcase = hexdigits.lower()
            self.char_set += ''.join(list(set(hexcase)))
            return
        if self.alpha:
            self.char_set += ascii_lowercase
        if self.upper:
            self.char_set += ascii_uppercase
        if self.numerals:
            self.char_set += digits
        if self.special:
            self.char_set += punctuation

    def generate(self):
        '''Generate the password using instance attributes.'''
        # TODO: This needs much improvement. As is, there is no assurance
        # that we have at least one character from each selected character
        # set included. This will lessen strength of passwords.
        self.build_character_set()
        password = ''
        chars_remaining = self.length
        while chars_remaining > 0:
            password += choice(self.char_set)
            chars_remaining -= 1
        self._password = password



from setuptools import setup, find_packages
from os import path


# Read the README into memory, include into package info
cwd = path.abspath(path.dirname(__file__))
with open(path.join(cwd, 'README.md'), 'r') as fh:
    long_description = fh.read()


setup(name='pywrdgen',
      packages=find_packages(),
      version='0.1.0',
      author='ncdulo',
      license='MIT',
      url='https://github.com/ncdulo/pywrdgen',
      keywords='password pwgen generate generator cli utility tool',
      description='Generate (possibly) secure passwords.',
      long_description=long_description,
      long_description_content_type='text/markdown',
      python_requires='>=3.6',
      install_requires=[
              'click',
          ],
      entry_points={
              'console_scripts': [
                  'pywrdgen = pywrdgen.__main__:main'
              ],
          },
      classifiers=[
              'Development Status :: 3 - Alpha',
              'Environment :: Console',
              'Intended Audience :: End Users/Desktop',
              'Intended Audience :: System Administrators',
              'License :: OSI Approved :: MIT License',
              'Natural Language :: English',
              'Operating System :: OS Independent',
              'Operating System :: POSIX :: Linux',
              'Programming Language :: Python :: 3',
              'Programming Language :: Python :: 3.6',
              'Programming Language :: Python :: 3.7',
              'Programming Language :: Python :: 3.8',
              'Programming Language :: Python :: 3 :: Only',
              'Topic :: Security',
              'Topic :: System :: Systems Administration',
              'Topic :: Utilities',
          ]
      )
